from urllib import request
from typing import Optional, List, Tuple
from dataclasses import dataclass
import os
import feedparser

from guessit import guessit
import inquirer

from nyaacli.colors import red, yellow, green, PromptTheme
from nyaacli.utils import clear_screen


def get_file_extension(path: str) -> str:
    """
    Gets the File extension from the path to a file

    'asd.txt' -> '.txt'
    'asd' -> ''
    '/path/to/asd.mkv' -> '.mkv'
    """
    filename, extension = os.path.splitext(path)

    return extension


def text_break() -> None:
    print('-' * 10, end='\n')


@dataclass
class Entry:
    title: str
    link: str
    original_title: str
    extension: str
    full_title: Optional[str]
    episode: Optional[int]
    other: Optional[str]
    release_group: Optional[str]
    screen_size: Optional[str]
    alternative_title: Optional[str]
    display_title: Optional[str]
    season: Optional[str]
    seeders: Optional[str]
    size: Optional[str]
    episode_title: Optional[str]


def search_torrent(search: str, episode: Optional[int] = None, dub: bool = False) -> Optional[Tuple[str, str]]:
    """
    Results a tuple with (Path to .torrent file, Result name of video file)
    Nyaa.si rss search flags
    https://github.com/nyaadevs/nyaa/blob/a38e5d5b53805ecb1d94853d849826f948f07aad/nyaa/views/main.py#L65
    """

    search_query = f"{search}".replace(' ', '%20')
    if episode:
        search_query += f" {episode}".replace(' ', '%20')

    # Parse Nyaa.si rss feed search
    feed: feedparser.FeedParserDict = feedparser.parse(f"https://nyaa.si/rss?c=1_2&q={search_query}&s=seeders&o=desc")

    entries: List[Entry] = []

    for entry in feed['entries']:
        title = guessit(entry['title'])

        if not title.get('screen_size'):
            # Ignore entries without Screen size property
            continue

        if not dub and 'dub' in entry['title'].lower():
            # Ignore entries with 'dub' in their titles if dub=False
            continue

        # Screen size needs to be higher than 480p
        good_size = int(title.get('screen_size').replace('p', '')) > 480

        if title.get('episode') == episode and title.get('type') == 'episode' and good_size:
            entries.append(Entry(
                link=entry['link'],
                size=entry['nyaa_size'],
                original_title=entry['title'],
                display_title=entry['title'],
                extension=get_file_extension(entry['title']),
                title=title.get('title'),
                season=title.get('season'),
                seeders=entry.get('nyaa_seeders'),
                full_title='',
                episode=title.get('episode'),
                episode_title=title.get('episode_title'),
                other=title.get('other'),
                release_group=title.get('release_group'),
                screen_size=title.get('screen_size'),
                alternative_title=title.get('alternative_title')
            ))

    if not entries:
        print(red(f'No results found for search: \'{search_query.replace("%20", " ")}\''))
        return None

    for entry in entries[:5]:
        entry_title = entry.title
        entry.full_title = entry.title

        if entry.alternative_title:
            entry_title += f" - {entry.alternative_title}"
            entry.full_title += f" - {entry.alternative_title}"

        if entry.episode or entry.episode_title:
            if entry.episode:
                entry_title += f" - Episode {entry.episode}"
                entry.full_title += f" - Episode {entry.episode}"
            else:
                entry_title += f" - Episode {entry.episode_title}"
                entry.full_title += f" - Episode {entry.episode_title}"

        if entry.season:
            entry_title += f" - Season {entry.season}"
            entry.full_title += f" - Season {entry.season}"

        if entry.release_group:
            entry_title += f" ({entry.release_group})"

        if entry.screen_size:
            entry_title += f" {entry.screen_size}"

        if entry.seeders:
            seeders = f'{entry.seeders} Seeders'

            if int(entry.seeders) > 40:
                entry_title += f" - {green(seeders)}"
            elif 40 > int(entry.seeders) > 20:
                entry_title += f" - {yellow(seeders)}"
            else:
                entry_title += f" - {red(seeders)}"

        if entry.size:
            entry_title += f" - {entry.size}"

        entry.display_title = entry_title

    questions = [
        inquirer.List(
            'entry',
            message=green("Select one of the entries below"),
            choices=[(str(entry.display_title), index) for index, entry in enumerate(entries[:5])],
        ),
    ]

    answer = inquirer.prompt(questions, theme=PromptTheme())

    clear_screen()

    index_choice = answer['entry'] - 1

    entry_choice = entries[index_choice]

    final_path = entry_choice.full_title

    torrent_path = f'/tmp/{final_path}.torrent'

    print(f"{green('[Downloading]')} '{torrent_path}'")

    request.urlretrieve(entry_choice.link, torrent_path)

    return torrent_path, final_path + entry_choice.extension



import os


def clear_screen():
    """
    Runs either 'cls' or 'clear' on the Terminal, 'cls' to clear Terminal on windows,
    which will ignore the rest of the command, and 'clear' to clear on Linux/Mac OS
    """
    os.system('cls||clear')



from inquirer.themes import Theme
from blessings import Terminal
from colorama import init

init()


def red(text: str) -> str:
    return f'\033[31m{text}\033[0m'


def green(text: str) -> str:
    return f'\33[92m{text}\033[0m'


def yellow(text: str) -> str:
    return f'\33[33m{text}\033[0m'


def blue(text: str) -> str:
    return f'\33[34m{text}\033[0m'


class PromptTheme(Theme):
    def __init__(self):
        super(PromptTheme, self).__init__()

        term = Terminal()

        self.Question.mark_color = term.yellow
        self.Question.brackets_color = term.normal
        self.Question.default_color = term.normal
        self.Editor.opening_prompt_color = term.bright_black
        self.Checkbox.selection_color = term.blue
        self.Checkbox.selection_icon = '❯'
        self.Checkbox.selected_icon = '◉'
        self.Checkbox.selected_color = term.yellow + term.bold
        self.Checkbox.unselected_color = term.normal
        self.Checkbox.unselected_icon = '◯'
        self.List.selection_color = term.blue
        self.List.selection_cursor = '❯'
        self.List.unselected_color = term.normal



import os
import sys
import time

import libtorrent

from nyaacli.colors import red, green
from nyaacli.utils import clear_screen


def download_torrent(filename: str, result_filename: str = None, show_progress: bool = True, base_path: str = 'Anime'):
    session = libtorrent.session({'listen_interfaces': '0.0.0.0:6881'})

    base_path = os.path.expanduser(base_path)

    info = libtorrent.torrent_info(filename)

    handle: libtorrent.torrent_handle = session.add_torrent({
        'ti': info,
        'save_path': base_path
    })

    status: libtorrent.session_status = handle.status()

    print(f"\n{green('◉ Started downloading torrent:')} {status.name}", end='\n\n')

    if show_progress:
        while not status.is_seeding:
            status = handle.status()

            progress = green(f"{status.progress * 100:.2f}%")
            download_rate = green(f"{status.download_rate / 1000:.1f} kB/s")

            sys.stdout.write(
                f'\r◉ {green(str(status.state).title())} - {progress} '
                f'(Download: {download_rate} - Upload: {status.upload_rate / 1000:.1f} kB/s - '
                f'Peers: {status.num_peers}) '
            )

            alerts = session.pop_alerts()

            alert: libtorrent.alert
            for alert in alerts:
                if alert.category() & libtorrent.alert.category_t.error_notification:
                    sys.stdout.write(f"\r{red('[Alert]')} {alert} ")

            sys.stdout.flush()

            time.sleep(1)

        print(green(f'\n\nFinished downloading {handle.name()}'), end='\n\n')

    if result_filename:
        old_name = f'{base_path}/{status.name}'
        new_name = f'{base_path}/{result_filename}'

        os.rename(old_name, new_name)

        clear_screen()

        print(f'Finished download at: \'{green(new_name)}\' ')


if __name__ == '__main__':
    if len(sys.argv) < 2:
        print('You need to pass in the .torrent file path.')
        sys.exit(1)

    download_torrent(sys.argv[1])



#!/usr/bin/env python

import sys

import click

from nyaacli.nyaa_search import search_torrent
from nyaacli.colors import red, green


@click.command()
@click.argument('anime')
@click.argument('episode', type=int, default=None, required=False)
@click.option('--output', '-o', default='~/Videos/Anime', help=green('Output Folder'), type=click.Path(), show_default=True)
def main(anime: str, episode: int, output: str):
    """
    Search for an Anime on https://nyaa.si and downloads it

    \b
    Usage:
        \33[92mnyaa \33[36m"Anime Name" \33[33m<Episode Number (Optional)> \33[34m-o <Output Folder (Default = '~/Videos/Anime')>\033[0m

    \b
    Example:
        \33[92mnyaa \33[36m"Kimetsu no Yaiba" \33[33m19 \33[34m-o "My/Animes/Folder/Kimetsu_No_Yaiba/\033[0m
    """
    torrent_search = search_torrent(anime, episode)

    if torrent_search:
        torrent_path, result_name = torrent_search

        download_torrent(torrent_path, result_name, base_path=output)


try:
    from nyaacli.torrenting import download_torrent
except ModuleNotFoundError:
    print(red("You need to have the 'python3-libtorrent' library installed to run this.\n"))
    print("If you use the apt package manager, you can install with:", green("sudo apt install python3-libtorrent"))

    libtorrent_url = "https://github.com/arvidn/libtorrent/blob/RC_1_2/docs/python_binding.rst"

    print(f"Otherwise, look into how you can build it here: {libtorrent_url}")
    sys.exit(1)

main()



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['nyaacli', 'nyaacli.cli']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.0,<8.0',
 'colorama>=0.4.1,<0.5.0',
 'feedparser>=5.2,<6.0',
 'guessit>=3.0,<4.0',
 'inquirer>=2.6,<3.0']

extras_require = \
{':python_version >= "3.6" and python_version < "3.7"': ['dataclasses>=0.7,<0.8']}

entry_points = \
{'console_scripts': ['nyaa = nyaacli.cli:main', 'nyaa-cli = nyaacli.cli:main']}

setup_kwargs = {
    'name': 'nyaacli',
    'version': '0.1.3',
    'description': 'A CLI for downloading Anime from https://nyaa.si',
    'long_description': '# Nyaa-cli [![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT) [![nyaacli](https://img.shields.io/pypi/pyversions/nyaacli)](https://pypi.org/project/nyaacli/) [![nyaa.si](https://img.shields.io/badge/-nyaa.si-green)](https://nyaa.si)\n\n\nA CLI for downloading Anime from https://nyaa.si making use of their RSS Feed and [python-libtorrent](https://github.com/arvidn/libtorrent/blob/RC_1_2/docs/python_binding.rst)\n\n---\n\n![image](https://user-images.githubusercontent.com/37747572/69002323-bb2ea100-08cb-11ea-9b47-20bd9870c8c0.png)\n\n---\n\n![image](https://user-images.githubusercontent.com/37747572/69002293-33e12d80-08cb-11ea-842e-02947726185d.png)\n\n---\n\n![image](https://user-images.githubusercontent.com/37747572/69002363-ad2d5000-08cc-11ea-9360-76bf1598512d.png)\n\n---\n\n## Installing\n\n- `python3 -m pip install nyaacli --user`\n  - *Note:* python-libtorrent will still need to be downloaded separately as shown below\n\n- This Program depends on python3-libtorrent, which can be installed using Apt with `sudo apt install python3-libtorrent` or can be built from source here: [python-libtorrent](https://github.com/arvidn/libtorrent/blob/RC_1_2/docs/python_binding.rst)\n\n---\n\n## Usage\n\n- **Help:** `nyaa --help` or `nyaa-cli --help`\n\n- `nyaa "Anime Name" <Episode Number (Optional)> -o <Output Folder (Default: ~/Videos/Anime)>`\n  - **Example:**\n    ```bash\n    # Downloading Episode 14 of \'Steins;gate\' to \'~/Anime/Steins;Gate\'\n    nyaa "Steins;Gate" 14 -o ~/Anime/Steins\\;Gate\n    ```\n  - Then select the entry you want to Download\n',
    'author': 'John Victor',
    'author_email': 'johnvictorfs@gmail.com',
    'url': 'https://github.com/johnvictorfs/nyaa-cli',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
